%Postprocess the results of the moss growth program. The directory in
%question should have all moss plant images with filenames 'stepi-j.png',
%where 'i' is the stage of growth and 'j' denotes which plant it belongs
%to.

function postprocess()
    steps = input('How many steps were these simulations run for?\n');
    steps = steps + 1;
    replicas = input('How many plants were simulated?\n');
    res = 'results';
    mkdir(res);
    count = 0;
    tic
	for len = 20:20:141
        for wid = 40:20:161
            for cur = [1,15,30,45,60,75,90]
                for bra = 0:15:91
                    dataDir = strcat('length',int2str(len),'width',int2str(wid),'curve',int2str(cur),'branch',int2str(bra));
    %                 area = zeros(steps, 1);
    %                 solidity = zeros(steps, 1);
    %                 eccentricity = zeros(steps, 1);
    %                 circularity = zeros(steps, 1);
    %                 fractalDimension = zeros(steps, 1);
                    %radiusGyration = zeros(steps, 1);
                    %pairCorrelation = zeros(steps, 500);
                    results = zeros(steps,replicas,5);

                    for i = 1:replicas
                        for j = 1:steps
                            displayString = strcat('Working on step ', int2str(j-1), ' of replica ', int2str(i-1),' of dataset ', dataDir,'time=',int2str(toc),'Percent done=',int2str(count/2401));
                            displayStringLength = length(displayString);
                            disp(displayString);

                            picture = imread(strcat(dataDir, '/step-', int2str(j-1), '-', int2str(i-1), '.png'));
                            picture = im2bw(picture(:,:,1), 0.5);
                            stats = regionprops(picture, 'image', 'area', 'solidity', 'eccentricity', 'perimeter', 'centroid');
                            picture = stats.Image;

    %                         area(j) = area(j) + stats.Area;
    %                         solidity(j) = solidity(j) + stats.Solidity;
    %                         eccentricity(j) = eccentricity(j) + stats.Eccentricity;
    %                         circularity(j) = circularity(j) + 4 * pi * stats.Area / (stats.Perimeter ^ 2);
    %                         fractalDimension(j) = fractalDimension(j) + fractal(picture);
    %                         %radiusGyration(j) = radiusGyration(j) + gyration(picture, stats);
                            results(j,i,1) =stats.Area;
                            results(j,i,2) =stats.Solidity;
                            results(j,i,3) =stats.Eccentricity;
                            results(j,i,4) =4 * pi * stats.Area / (stats.Perimeter ^ 2);
                            results(j,i,5) =fractal(picture);
                            %results(j,i,6) =gyration(picture, stats);

                            %pairCorrTemp = paircorrelation(picture, stats);
                            %for k = 1:500
                            %    pairCorrelation(j, k) = pairCorrelation(j,k) + pairCorrTemp(k);
                            %end

                            for l = 0:displayStringLength
                                fprintf('\b');
                            end
                        end
                    end

    %                 for j = 1:steps
    %                     area(j) = area(j) / replicas;
    %                     solidity(j) = solidity(j) / replicas;
    %                     eccentricity(j) = eccentricity(j) / replicas;
    %                     circularity(j) = circularity(j) / replicas;
    %                     fractalDimension(j) = fractalDimension(j) / replicas;
    %                     %radiusGyration(j) = radiusGyration(j) / replicas;
    % 
    %                     %for k = 1:500
    %                     %    pairCorrelation(j, k) = pairCorrelation(j, k) / replicas;
    %                     %end
    %                 end

    %                 dlmwrite(strcat(resultsDir, 'area.txt'), area, '\t');
    %                 dlmwrite(strcat(resultsDir, 'solidity.txt'), solidity, '\t');
    %                 dlmwrite(strcat(resultsDir, 'eccentricity.txt'), eccentricity, '\t');
    %                 dlmwrite(strcat(resultsDir, 'circularity.txt'), circularity, '\t');
    %                 dlmwrite(strcat(resultsDir, 'fractal_dimension.txt'), fractalDimension, '\t');
                    %dlmwrite(strcat(resultsDir, 'radius_of_gyration.txt'), radiusGyration, '\t');
                    %dlmwrite(strcat(resultsDir, 'pair_correlation.txt'), pairCorrelation, '\t');
                    save(strcat(res,'\',dataDir,'.mat'),'results');
                end
            end
        end
	end
end

function f = fractal(picture)
    xMax = size(picture, 1);
    yMax = size(picture, 2);
    if (xMax > yMax)
        iterations = floor(log(xMax)/log(2));
        maxSize = xMax;
    else
        iterations = floor(log(yMax)/log(2));
        maxSize = yMax;
    end

    N = zeros(iterations, 1);
    s = zeros(iterations, 1);

    for i = 1:iterations
        numFilled = 0;
        numSideBox = 2.0 ^ (i-1);
        boxSize = floor(maxSize / numSideBox);
        s(i) = 1.0 / boxSize;
        for a = 1:numSideBox
            for b = 1:numSideBox
                filled = false;
                for c = 1:boxSize
                    for d = 1:boxSize
                        if ~filled
                            x = c + (a-1) * boxSize;
                            y = d + (b-1) * boxSize;
                            if (x < xMax && y < yMax && picture(x,y))
                                filled = true;
                            end
                        end
                    end
                end
                if filled
                    numFilled = numFilled + 1;
                end
            end
        end
        N(i) = N(i) + numFilled;
    end
    p = polyfit(log(s), log(N), 1);

    f = p(1);
end
    
function f = gyration(picture, stats)
    sum = 0;
    num = 0;
    for i = 1:size(picture, 1)
        for j = 1:size(picture, 2)
            if picture(i,j) == 1
                num = num + 1;
                dx = i - stats.Centroid(1);
                dy = j - stats.Centroid(2);
                val = dx^2 + dy^2;
                sum = sum + val;
            end
        end
    end
    
    f = sqrt(sum / num);
end

function f = paircorrelation(picture, stats)
    maxBin = 500;
    xMax = size(picture, 1);
    yMax = size(picture, 2);
    bins = zeros(maxBin, 1);
    totalPairs = 0;


    N = stats.Area;
    rho = stats.Area / (xMax * yMax);

    % Make a list of all the relevant pixel locations
    list = zeros(stats.Area, 2);
    iter = 0;
    for i = 1:xMax
        for j = 1:yMax
            if picture(i,j)
                list(iter + 1, 1) = i;
                list(iter + 1, 2) = j;
                iter = iter + 1;
            end
        end
    end

    % Make a histogram of the distances between each pair of pixels
    for i = 1:iter
        for j = 1:iter
            dx = list(i,1) - list(j,1);
            dy = list(i,2) - list(j,2);
            dist = floor(sqrt(dx^2 + dy^2)) + 1;
            if dist < maxBin
                bins(dist) = bins(dist) + 1;
                totalPairs = totalPairs + 1;
            end
        end
    end

    % Normalize the histogram by shell area and number of pixels
    % To also normalize by density, add rho here
    for i = 1:maxBin
        bins(i) = bins(i) / (2 * pi * i * N);
    end

    f = bins;
end